@@ -1,4 +1,4 @@
 from auri.command_line import cli
 
 if __name__ == "__main__":
-    cli()
\ No newline at end of file
+    cli(prog_name="auri")
\ No newline at end of file
