@@ -1,10 +1,10 @@
 import json
-import sys
 from difflib import get_close_matches
+from typing import Union
 
 import click
 
-from auri.ambilight_controller import AmbilightController
+from auri.ambilight_controller import AmbilightController, AMBILIGHT_EFFECT_NAME
 from auri.aurora import Aurora, AuroraException
 from auri.device_finder import DeviceFinder
 from auri.device_manager import DeviceManager
@@ -13,13 +13,144 @@ from auri.device_manager import DeviceManager
 # TODO create wrapper or validator that checks we have a valid default/named aurora and redirects to setup if needed
 # TODO catch config and aurora exceptions and print them nicely
 
+class CtxObj:
+    def __init__(self, aurora: Union[str, None], verbose: bool = False):
+        self.dm = DeviceManager(verbose=verbose)
+        self.aurora_name = aurora
+        self.__aurora = None
+        self.verbose = verbose
+
+    @property
+    def aurora(self):
+        """Lazily create the actual aurora object when it's needed"""
+        if not self.__aurora:
+            self.__aurora = self.dm.get_by_name_or_active(self.aurora_name)
+        return self.__aurora
+
+
 @click.group()
-@click.version_option("1.2.2")
-def cli():
-    pass
+@click.option("-a", "--aurora", default=None, help="Which Nanoleaf to use, see `device list`")
+@click.option("-v", "--verbose", is_flag=True, default=False, help="More Logging")
+@click.pass_context
+@click.version_option("1.3.4")
+def cli(ctx: CtxObj, aurora: Union[str, None], verbose: bool):
+    ctx.obj = CtxObj(aurora, verbose)
 
 
-# AURORA AND CONTEXT MANAGEMENT
+# GENERAL SETTING COMMANDS
+
+@cli.command(name="on", help="Turn on the active device")
+@click.pass_obj
+def on_command(obj: CtxObj):
+    obj.aurora.on = True
+
+
+@cli.command(name="off", help="Turn off the active device")
+@click.pass_obj
+def off_command(obj: CtxObj):
+    obj.aurora.on = False
+
+
+@cli.command(name="brighter", help="Make the current effect brighter")
+@click.pass_obj
+def brighter_command(obj: CtxObj):
+    current = obj.aurora.brightness
+    new = min(current + 20, 100)
+    click.echo(f"Brightness {current}->{new}")
+    obj.aurora.brightness = new
+
+
+@cli.command(name="darker", help="Make the current effect darker")
+@click.pass_obj
+def darker_command(obj: CtxObj):
+    current = obj.aurora.brightness
+    new = max(current - 20, 0)
+    click.echo(f"Brightness {current}->{new}")
+    obj.aurora.brightness = new
+
+# EFFECT COMMANDS
+
+@cli.command(name="play", help="Switches the device to a specific effect. Uses spelling correction.")
+@click.argument("name", nargs=-1)
+@click.pass_obj
+def play_command(obj: CtxObj, name: str):
+    effect_name = " ".join(name)
+    closest = get_close_matches(effect_name, obj.aurora.get_effect_names(), n=1, cutoff=0)
+    if len(closest) == 0:
+        # As long as there is a single effect, this should not happen
+        click.echo(f"Did not find anything similar to {effect_name}, are there no effects on this device?")
+        return
+    effect_name = closest[0]
+
+    if effect_name.lower() == AMBILIGHT_EFFECT_NAME.lower():
+        # TODO: could probably also forward this to ambi automatically
+        click.echo("WARNING: Playing AuriAmbi doesn't activate the Ambi function, use `auri ambi` instead!")
+
+    effect = obj.aurora.get_effect_by_name(effect_name)
+    obj.aurora.set_active_effect(effect_name)
+    click.echo(f"Set current effect to {effect_name} {effect.color_flag_terminal()}")
+
+
+@cli.command(name="delete", help="Deletes a specified effect. Warning: This isn't reversible!")
+@click.argument("name", nargs=-1)
+@click.pass_obj
+def delete_command(obj: CtxObj, name: str):
+    effect_name = " ".join(name)
+    click.confirm(f"This will delete the effect '{effect_name}' from {obj.aurora}, are you sure?", abort=True)
+
+    try:
+        obj.aurora.delete_effect(effect_name)
+        click.echo(f"Deleted effect {effect_name}")
+    except AuroraException:
+        click.echo(f"Did not find effect with name {effect_name}")
+
+
+@cli.command(name="list", help="Displays all effects that are currently installed on this device")
+@click.option("-n", "--names", is_flag=True, default=False, help="Only prints the effect names and exits")
+@click.pass_obj
+def list_command(obj: CtxObj, names: bool):
+    active_effect_name = obj.aurora.get_active_effect_name()
+    for effect in obj.aurora.get_effects():
+
+        # simple name printing
+        if names:
+            click.echo(effect.name)
+            continue
+
+        # pretty printing with effect colors and active marker
+        active = "[X]" if active_effect_name == effect.name else "[ ]"
+        click.echo(f"{active} {effect.color_flag_terminal()} {effect.name}")
+
+
+# AMBINANO COMMANDS
+
+@cli.command(name="ambi", help="Toggles the ambilight functionality")
+@click.pass_context
+def ambi(ctx: click.Context):
+    ambi_controller = AmbilightController(ctx.obj.aurora, verbose=ctx.obj.verbose)
+
+    if not ambi_controller.is_running:
+        ambi_controller.start()
+        click.echo("auri ambi started")
+        return
+
+    ambi_controller.stop()
+    ctx.forward(off_command)
+    click.echo("auri ambi stopped")
+
+
+@cli.command(name="ambi-blocking", help="Toggles the ambilight functionality", hidden=True)
+@click.pass_obj
+def ambi_blocking(obj: CtxObj):
+    """This is used to start the ambi in blocking mode
+
+    Not exposed to the user, it needs to match ambilight_controller.AMBI_CALL_ARGS to identify the process
+    """
+    ambi_controller = AmbilightController(obj.aurora, verbose=obj.verbose)
+    ambi_controller.start_blocking()
+
+
+# DEVICE AND CONTEXT MANAGEMENT
 
 
 @cli.group(name="device", help="Interact with Nanoleaf devices")
@@ -29,49 +160,25 @@ def device_group():
 
 @device_group.command(name="activate", help="Set a specified Nanoleaf to the currently active one")
 @click.argument("name", nargs=-1)
-@click.option("-v", "--verbose", is_flag=True, default=False, help="More Logging")
-def activate(name: str, verbose: bool):
+@click.pass_obj
+def activate(obj: CtxObj, name: str):
     name = " ".join(name)
-    DeviceManager(verbose=verbose).set_active(name)
+    obj.dm.set_active(name)
     click.echo(f"Set {name} as active Aurora")
 
 
 @device_group.command(name="list", help="Lists all currently configured Nanoleaf devices")
-@click.option("-v", "--verbose", is_flag=True, default=False, help="More Logging")
-def device_list_command(verbose: bool):
-    manager = DeviceManager(verbose=verbose)
-    for aurora in manager.get_all():
-        active = "[X]" if manager.is_active(aurora) else "[ ]"
+@click.pass_obj
+def device_list_command(obj: CtxObj):
+    for aurora in obj.dm.get_all():
+        active = "[X]" if obj.dm.is_active(aurora) else "[ ]"
         click.echo(f"{active} {str(aurora)}")
 
 
-@device_group.command(name="query", help="Query the Nanoleaf device with a custom attribute for debugging")
-@click.argument("option", default="info")
-@click.option("-a", "--aurora", default=None, help="Which Nanoleaf to use")
-@click.option("-v", "--verbose", is_flag=True, default=False, help="More Logging")
-def device_query_command(option: str, aurora: str, verbose: bool):
-    aurora = DeviceManager(verbose=verbose).get_by_name_or_active(aurora)
-    try:
-        result = getattr(aurora, option)
-        if callable(result):
-            result = result()
-        click.echo(str(result))
-    except AttributeError:
-        click.echo(f"Operator '{option}' doesn't exist")
-
-
-@device_group.command(name="on", help="Turn on the active device")
-@click.option("-a", "--aurora", default=None, help="Which Nanoleaf to use")
-@click.option("-v", "--verbose", is_flag=True, default=False, help="More Logging")
-def device_on_command(aurora: str, verbose: bool):
-    DeviceManager(verbose=verbose).get_by_name_or_active(aurora).on = True
-
-
-@device_group.command(name="off", help="Turn off the active device")
-@click.option("-a", "--aurora", default=None, help="Which Nanoleaf to use")
-@click.option("-v", "--verbose", is_flag=True, default=False, help="More Logging")
-def device_off_command(aurora: str, verbose: bool):
-    DeviceManager(verbose=verbose).get_by_name_or_active(aurora).on = False
+@device_group.command(name="identify", help="Make the device blink")
+@click.pass_obj
+def device_identify_command(obj: CtxObj):
+    obj.aurora.identify()
 
 
 @device_group.command(name="setup", help="Run this to configure new Nanoleaf devices")
@@ -114,130 +221,6 @@ def device_setup_command(amount: int, verbose: bool):
     click.echo("Added all requested Auroras - Done.")
 
 
-# EFFECT MANAGEMENT
-
-
-@cli.group(name="effects", help="Interact with effects of the current device")
-def effects_group():
-    pass
-
-
-@effects_group.command(name="play", help="Switches the device to a specific effect. Uses spelling correction.")
-@click.argument("name", nargs=-1)
-@click.option("-a", "--aurora", default=None, help="Which Nanoleaf to use")
-@click.option("-v", "--verbose", is_flag=True, default=False, help="More Logging")
-def effects_play_command(name: str, aurora: str, verbose: bool):
-    aurora = DeviceManager(verbose=verbose).get_by_name_or_active(aurora)
-    effect_name = " ".join(name)
-
-    if effect_name.lower() == "auriambi":
-        # TODO: could probably also forward this to ambi automatically
-        click.echo("WARNING: Playing AuriAmbi doesn't activate the Ambi function, use `auri effects ambi` instead!")
-
-    closest = get_close_matches(effect_name, aurora.get_effect_names(), n=1, cutoff=0)
-    if len(closest) == 0:
-        # As long as there is a single effect, this should not happen
-        click.echo(f"Did not find anything similar to {effect_name}, are there no effects on this device?")
-        return
-    effect_name = closest[0]
-    effect = aurora.get_effect_by_name(effect_name)
-    aurora.set_active_effect(effect_name)
-    click.echo(f"Set current effect to {effect_name} {effect.color_flag_terminal()}")
-
-
-@effects_group.command(name="delete", help="Deletes a specified effect. Warning: This isn't reversible!")
-@click.argument("name", nargs=-1)
-@click.option("-a", "--aurora", default=None, help="Which Nanoleaf to use")
-@click.option("-v", "--verbose", is_flag=True, default=False, help="More Logging")
-def effects_delete_command(name: str, aurora: str, verbose: bool):
-    aurora = DeviceManager(verbose=verbose).get_by_name_or_active(aurora)
-    effect_name = " ".join(name)
-    click.confirm(f"This will delete the effect '{effect_name}' from {aurora}, are you sure?", abort=True)
-
-    try:
-        aurora.delete_effect(effect_name)
-        click.echo(f"Deleted effect {effect_name}")
-    except AuroraException:
-        click.echo(f"Did not find effect with name {effect_name}")
-
-
-@effects_group.command(name="get", help="Gets either name or brightness of the current effect")
-@click.argument("option")
-@click.option("-a", "--aurora", default=None, help="Which Nanoleaf to use")
-@click.option("-v", "--verbose", is_flag=True, default=False, help="More Logging")
-def effects_get_command(option: str, aurora: str, verbose: bool):
-    aurora = DeviceManager(verbose=verbose).get_by_name_or_active(aurora)
-    output = f"Couldn't find option {option} for `effects get"
-    if option == "name":
-        output = aurora.get_active_effect_name()
-    elif option == "brightness":
-        output = aurora.brightness
-
-    click.echo(output)
-
-
-@effects_group.command(name="set", help="Sets either name or brightness of the current effect")
-@click.argument("option")
-@click.argument("value")
-@click.option("-a", "--aurora", default=None, help="Which Nanoleaf to use")
-@click.option("-v", "--verbose", is_flag=True, default=False, help="More Logging")
-def effects_set_command(option: str, value: str, aurora: str, verbose: bool):
-    aurora = DeviceManager(verbose=verbose).get_by_name_or_active(aurora)
-    if option == "brightness":
-        aurora.brightness = int(value)
-    elif option == "identify":
-        aurora.identify()
-    else:
-        click.echo(f"Couldn't find the option {option}")
-        sys.exit(1)
-
-    click.echo(f"Set {option} to {value}")
-
-
-@effects_group.command(name="list", help="Displays all effects that are currently installed on this device")
-@click.option("-a", "--aurora", default=None, help="Which Nanoleaf to use")
-@click.option("-n", "--names", is_flag=True, default=False, help="Only prints the effect names and exits")
-@click.option("-v", "--verbose", is_flag=True, default=False, help="More Logging")
-def effects_list_command(aurora: str, names: bool, verbose: bool):
-    aurora = DeviceManager(verbose=verbose).get_by_name_or_active(aurora)
-    active_effect_name = aurora.get_active_effect_name()
-    for effect in aurora.get_effects():
-
-        # simple name printing
-        if names:
-            click.echo(effect.name)
-            continue
-
-        # pretty printing with effect colors and active marker
-        active = "[X]" if active_effect_name == effect.name else "[ ]"
-        click.echo(f"{active} {effect.color_flag_terminal()} {effect.name}")
-
-
-@cli.group(name="ambi", help="Manage the ambilight functionality")
-def ambi_group():
-    pass
-
-
-# TODO the additional arguments only work when being run in blocking mode, move them to the config
-@ambi_group.command(name="start", help="Activates the ambilight functionality")
-@click.option("-a", "--aurora", default=None, help="Which Nanoleaf to use")
-@click.option("-v", "--verbose", is_flag=True, default=False, help="More Logging")
-@click.option("-b", "--block", is_flag=True, default=False, help="Block the shell while running ambi")
-def effects_ambi_start(aurora: str, verbose: bool, block: bool):
-    aurora = DeviceManager(verbose=verbose).get_by_name_or_active(aurora)
-    AmbilightController(aurora, verbose=verbose).start(blocking=block)
-    click.echo("auri ambi started")
-
-
-@ambi_group.command(name="stop", help="Stops the ambilight functionality")
-@click.option("-a", "--aurora", default=None, help="Which Nanoleaf to use")
-@click.option("-v", "--verbose", is_flag=True, default=False, help="More Logging")
-def effects_ambi_stop(aurora: str, verbose: bool):
-    aurora = DeviceManager(verbose=verbose).get_by_name_or_active(aurora)
-    AmbilightController(aurora, verbose=verbose).stop()
-    click.echo("auri ambi stopped")
-
-
 # ALFRED WORKFLOW HELPERS
 
 @cli.group(name="alfred", help="Alfred integration functions")
@@ -269,17 +252,16 @@ def alfred_prompt_command():
 @alfred_group.command(name="command", help="Parse command from `auri alfred prompt`")
 @click.argument("command", nargs=-1)
 @click.pass_context
-def alfred_command_command(ctx, command: str):
+def alfred_command_command(ctx: click.Context, command: str):
     command_string = " ".join(command)
-    ctx.invoke(effects_set_command, name=command_string)
+    ctx.invoke(play_command, name=command_string)
 
 
 @alfred_group.command(name="images", help="Generates image files for each effect")
-@click.option("-v", "--verbose", is_flag=True, default=False, help="More Logging")
-def alfred_images_command(verbose: bool):
-    manager = DeviceManager(verbose=verbose)
-    manager.generate_image_cache()
-    click.echo(f"Generated images into {manager.image_path}")
+@click.pass_obj
+def alfred_images_command(obj: CtxObj):
+    obj.dm.generate_image_cache()
+    click.echo(f"Generated images into {obj.dm.image_path}")
 
 
 if __name__ == '__main__':
